#include "board.h"

void CBoard::initialize(int size)
{
	boardWidth=0;
	board=NULL;
	isFirst=true;
	setSize(size);
	for (int i=0; i<=size; i++)
	{
		for (int j=0; j<=size; j++)
		{
			if (i==0||j==0)
			{
				board[i][j].setBKGAll(DefaultIndexFontColor, DefaultIndexBKGroundColor);
								
				if (i!=0)
				{
					board[i][j].setNumber(i-1, false);
				}
				if (j!=0)
				{
					board[i][j].setNumber(j-1, true);
				}
			}
		}
	}
}

void CBoard::setValue(char** matrix)
{
	for (int i=1; i<=boardWidth; i++)
	{
		for (int j=1; j<=boardWidth; j++)
		{
			board[i][j].setValue(matrix[i-1][j-1]);
		}
	}
}

void CBoard::setSize(int size)
{
	if (boardWidth!=size)
	{
		if (board!=NULL)
		{
			for (int i=0; i<=boardWidth; i++)
			{
				delete []board[i];			
			}
		}
		delete[] board;
		boardWidth=size;
		board=new CCell*[size+1];
		for (int i=0; i<=size; i++)
		{			
			board[i]=new CCell[size+1];
		}		
	}
}
		
void CBoard::show()
{
	display(DefaultBoard_x, DefaultBoard_y);
}

CBoard::CBoard(int size)
{
	initialize(size);	
}

void CBoard::setValue(int r, int c, char ch)
{
	board[r+1][c+1].setValue(ch);
}

char CBoard::getValue(int r, int c)
{
	return board[r+1][c+1].getValue();
}

void CBoard::display(int x, int y)
{	
	for (int i=0; i<=boardWidth; i++)
	{
		for (int j=0; j<=boardWidth; j++)
		{
			board[i][j].display(x+2*j, y+i*2);
		}

	}
}

void CBoard::update(bool rollBack)
{
	int row=curRow, col=curCol;
	while (board[row][col].getValue()!=0)
	{
		if (rollBack)
		{
			if (!board[row][col].getStatus())
			{				
				board[row][col].setValue(DefaultCharacter);
			}
		}	
		board[row][col].setFontColor(DefaultFontColor);
		board[row][col].setStatus(true);
		if (isVer)
		{
			row++;
		}
		else
		{
			col++;
		}
		if (row>boardWidth||col>boardWidth)
		{
			break;
		}
	}
}

void CBoard::setStartPoint()
{
	headRow=curRow;
	headCol=curCol;
	if (isVer)
	{
		headRow=findHead(headRow, headCol, isVer);
	}
	else
	{
		headCol=findHead(headRow, headCol, isVer);
	}	
}

char* CBoard::verifyStr(int& exchangedIndex)
{
	setStartPoint();
	getFirstStr(exchangedIndex);
	isFirst=true;//set back!!!
	return buffer;
}


void CBoard::getFirstStr(int& exchangedIndex)
{
	int row=headRow, col=headCol, pos=0;
	while (board[row][col].getValue()!=0) 
	{
		buffer[pos]=board[row][col].getValue();
		if (exchRow==row&&exchCol==col)
		{
			exchangedIndex=pos;
		}
		pos++;
		if (isVer)
		{
			row++;
		}
		else
		{
			col++;
		}
		if (row==boardWidth||col==boardWidth)
		{
			break;
		}
	}
	/*
	//this is means that it is only a single char word
	//try to search if it is possible to have other word
	if (pos==1)
	{
		pos=0;
		if (isVer)
		{
			col=findHead(row, col, !isVer);
			do//the boardWidth is one more than the real size 
			{
				buffer[pos]=board[row][col].getValue();
				if (row==exchRow&&col==exchCol)
				{
					exchangedIndex=pos;
				}
				col++;
				pos++;
				if (col>boardWidth)
				{
					break;
				}
			} while (board[row][col].getValue()!=0);
			headRow++;
			//buffer[pos]='\0';
		}
		else
		{
			row=findHead(row, col, !isVer);
			do
			{
				buffer[pos]=board[row][col].getValue();
				if (row==exchRow&&col==exchCol)
				{
					//find the index of exchanged letter
					exchangedIndex=pos;
				}
				row++;
				pos++;
				if (row>boardWidth)
				{
					break;
				}
			}while (board[row][col].getValue()!=0);
			//buffer[pos]='\0';
			headCol++;
		
		}

	}
	*/

	buffer[pos]='\0';
}

char* CBoard::getNextStr(int& exchangedIndex)
{	
	if (isFirst)
	{
		setStartPoint();
		isFirst=false;
		getFirstStr(exchangedIndex);
		return buffer;//the original one will always return
	}
	if (doFindStr(exchangedIndex))
	{
		return buffer;
	}
	else
	{
		return NULL;
	}
}

bool CBoard::doFindStr(int& exchangedIndex)
{
	int row=headRow, col=headCol, pos=0;
	exchangedIndex=-1;
	if (headRow>boardWidth||headCol>boardWidth)
	{
		return false;
	}
	if (board[row][col].getValue()==0)
	{
		return false;
	}
	if (isVer)
	{
		//skip the old word placed by others
		if (board[row][col].getStatus())
		{
			headRow++;
			return doFindStr(exchangedIndex);
		}
		col=findHead(row, col, !isVer);//the crossing direction
		do//the boardWidth is one more than the real size 
		{
			buffer[pos]=board[row][col].getValue();
			if (row==exchRow&&col==exchCol)
			{
				exchangedIndex=pos;
			}
			col++;
			pos++;
			if (col>boardWidth)
			{
				break;
			}
		} while (board[row][col].getValue()!=0);
		buffer[pos]='\0';
		headRow++;
		if (pos>MinimumLengthOfWord)
		{
			return true;
		}
		else
		{				
			return doFindStr(exchangedIndex);
		}
	}
	else
	{
		if (board[row][col].getStatus())
		{
			headCol++;
			return doFindStr(exchangedIndex);
		}
		row=findHead(row, col, !isVer);
		do
		{
			buffer[pos]=board[row][col].getValue();
			if (row==exchRow&&col==exchCol)
			{
				//find the index of exchanged letter
				exchangedIndex=pos;
			}
			row++;
			pos++;
			if (row>boardWidth)
			{
				break;
			}
		}while (board[row][col].getValue()!=0);
		buffer[pos]='\0';
		headCol++;
		if (pos>=MinimumLengthOfWord)
		{
			return true;
		}
		else
		{			
			return doFindStr(exchangedIndex);
		}
	}
}

//to do
int CBoard::findHead(int startRow, int startCol, bool isVertical)
{
	if (isVertical)
	{
		while (startRow>0) //because the coord is starting from 1
		{
			if (board[startRow][startCol].getValue()==0)
			{
				break;
			}
			startRow--;
		}
		return startRow+1;
	}
	else
	{
		while (startCol>0)
		{
			if (board[startRow][startCol].getValue()==0)
			{
				break;
			}
			startCol--;
		}
		return startCol+1;
	} 
}



bool CBoard::placeWord(int r, int c, const char* str, bool isVertical, int index)
{
	int row=r+1, col=c+1, pos=0;
	curRow=row;
	curCol=col;
	isVer=isVertical;
	if (index!=-1)//do have a special exchanged letter!
	{
		exchRow=curRow;
		exchCol=curCol;
		isVertical?exchRow+=index:exchCol+=index;//the coord of exchanged letter
	}
	else
	{
		exchRow=exchCol=-1;//this ensures no coord will match
	}
	while (str[pos]!='\0')
	{
		if (board[row][col].getValue()==0)
		{
			board[row][col].setValue(str[pos]);
			board[row][col].setFontColor(DefaultHighLightFont);
			board[row][col].setStatus(false);
			pos++;
		}
		else
		{
			//the starting position cannot be occupied
			if (pos==0)
			{
				return false;
			}
			if (pos<=index)//if index==-1 never execute here
			{
				//update new exchanged letter coord
				isVertical?exchRow++:exchCol++;
			}
		}
		if (isVertical)
		{
			row++;
		}
		else
		{
			col++;
		}
		if (row>boardWidth||col>boardWidth)//the bound is boardWidth
		{			
			return str[pos]=='\0';
		}
	}
	return true;
}


		

