#include "control.h"
#include "scoreboard.h"

CDeck playingDeck, playedDeck(true);
CScoreBoard scoreBoard;
CBoard board;
CDictionary dictionary(false);
char* defaultInputFileName="c:\\readme.txt";


void CControl::playGame()
{
	//bool quit=false;
	int choice;
	initPlayGame();
	choice=listenMenu();
	while (true)
	{
		switch (choice)
		{
		case SELECTCARD:
			choice =doSelectCard();			
			break;
		case SELECTLETTER:
			choice=doSelectLetter();
			break;
		case TRADE:
			if (!player[turn].traded&&!player[turn].formed)
			{				
				choice =doTrade();
			}
			else
			{
				showMessage(PLAYERALREADYTRADED);
				choice=CANCEL;
			}
			break;
		case PICKCARD:
			if (!player[turn].picked)
			{
				choice=doPickCard();
			}
			else
			{
				showMessage(PLAYERALREADYPICKED);
				choice=CANCEL;
			}
			break;
		case FORMWORD:
			if (!player[turn].formed)
			{
				choice=doFormWord();
			}
			else
			{
				showMessage(PLAYERALREADYFORMED);
				choice=CANCEL;
			}
			break;
		case QUIT:
			return;
			break;
		case ENDROUND:
			choice=doEndRound();
			break;
		case CANCEL:
			choice =doCancel();
			break;
		case DISCARD:
			choice =doDiscard();
			break;
		default:
			choice=listenMenu();
			break;
		}
	}
}

void CControl::calculateScore()
{
	int points=0, index=0, score=0;
	char* ptr;
	int pos=0;
	//index=player[turn].rackLetter.exchangedIndex;
	points=player[turn].rackCard.points;
	while ((ptr=board.getNextStr(index))!=NULL)
	{
		if (!dictionary.findWord(ptr))
		{
			continue;
		}
		pos=0;
		while (ptr[pos]!='\0')
		{
			if (pos!=index)
			{
				score+=bag.getTokenPoint(ptr[pos]);
			}
			else
			{
				score+=bag.getTokenPoint(ptr[pos])+points;
			}
			pos++;
		}		
	}
	//here to set score to score board
	//this is for debug only
	scoreBoard.setPlayerScore(score);
	scoreBoard.show();
	//this is for debug only
	
}

int CControl::doDiscard()
{
	CToken temp;
	//show prompt message, this is no cancel operation
	if (player[turn].rackCard.selectedCount>0)
	{
		for (int i=0; i<player[turn].rackCard.selectedCount; i++)
		{
			temp=player[turn].rackCard.tokens[player[turn].rackCard.selected[i]];
			playedDeck.addCard(temp);
		}
		player[turn].rackCard.updateRack();
		playedDeck.show();
		player[turn].rackCard.show();
	}
	if (player[turn].rackLetter.selectedCount>0)
	{
		for (int i=0; i<player[turn].rackLetter.selectedCount; i++)
		{
			temp=player[turn].rackLetter.tokens[player[turn].rackLetter.selected[i]];
			bag.addLetter(temp.getValue());
		}
		bag.shuffle();
		bag.show();
		player[turn].rackLetter.updateRack();
		player[turn].rackLetter.show();
	}
	return listenMenu();
}


int CControl::doCancel()
{
	player[turn].rackCard.updateRack(true);
	player[turn].rackLetter.updateRack(true);
	player[turn].traded=false;
	player[turn].formed=false;
	player[turn].show();
	board.update(true);
	message.clearAll();
	
	//board.show();
	return listenMenu();
}

int CControl::doEndRound()
{
	showMessage(ENDROUND);
	board.update();

	player[turn].rackLetter.updateRack();
	player[turn].rackCard.updateRack();
	fillUpRack();
	//player[turn].show();
	
	//set scoreboard to show current player
	nextTurn();
	
	//update all view to show
	//fillUpRack();
	player[turn].traded=false;
	player[turn].formed=false;
	player[turn].picked=false;
	player[turn].rackCard.canRollBack=true;
	player[turn].rackLetter.canRollBack=true;
	scoreBoard.switchCurrentPlayer();
	message.clearAll();
	show();
	return listenMenu();
}

void CControl::nextTurn()
{
	turn=(turn+1)%2;//next player
}


int CControl::doFormWord()
{
	int row=-1, col=-1;
	int index;
	char dir;
	showMessage(FORMWORD);
	char* ptr;
	ptr=player[turn].rackLetter.exportWord();
	//this should be done after the word is placed on the board
	/*
	if (!dictionary.findWord(ptr))
	{
		showMessage(INVALIDWORDFORMED);
		return CANCEL;
	}
	*/

	//showMessage(INPUTPLACEWORDINFO);
	showMessage(INPUTROW);
	row=listenInt();
	if (row>=DefaultBoardSize||row<0)
	{
		showMessage(INVALIDROWCOLNUMBER);
		return CANCEL;		
	}
	
	
	showMessage(INPUTCOL);
	col=listenInt();
	if (col>=DefaultBoardSize||col<0)
	{
		showMessage(INVALIDROWCOLNUMBER);
		return CANCEL;		
	}
	showMessage(INPUTDIRECTION);
	dir=listenChar();

	//points=player[turn].rackCard.points;

	index=player[turn].rackLetter.exchangedIndex;

	//have no verification here now, but need future
	if (board.placeWord(row, col, ptr, dir=='V'||dir=='v', index))
	{
		int temp;
		//char* ptr;
		ptr=board.verifyStr(temp);
		if (!dictionary.findWord(ptr))
		{
			showMessage(INVALIDWORDFORMED);
			return CANCEL;
		}
		calculateScore();
		board.update();
		board.show();
		player[turn].rackLetter.canRollBack=false;
		player[turn].rackCard.canRollBack=false;
		player[turn].rackLetter.updateRack();		
		player[turn].rackLetter.show();	
		for (int i=0; i<player[turn].rackCard.selectedCount; i++)
		{
			CToken temp;
			player[turn].rackCard.removeToken(player[turn].rackCard.selected[i], temp);
			playedDeck.addCard(temp);
		}
		playedDeck.show();
		player[turn].rackCard.updateRack();
		player[turn].rackCard.show();
		player[turn].formed=true;
		message.clearWord();
	}
	else
	{
		showMessage(UNABLEPLACEWORD);
		return CANCEL;
		//board.update(true);
		//board.show();
	}
	return listenMenu();
}





//I want a simple solution: you can only pick card if your last selected card
//has been discarded
int CControl::doPickCard()
{
	CToken target, source;
	int& count=player[turn].rackCard.selectedCount;
	int index=player[turn].rackCard.selected[count-1];

	if (count>0)
	{
		target=player[turn].rackCard.tokens[index];
		if (target.getValue()==0)//empty space
		{
			if (playedDeck.removeCard(source))
			{
				//remove from selected count and update displaying
				player[turn].rackCard.tokens[index]=source;
				count--;
				/*
				if (player[turn].rackCard.tokens[
					player[turn].rackCard.selected[player[turn].rackCard.selectedCount-1]]
				player[turn].rackCard.tokens[
					player[turn].rackCard.selected[--player[turn].rackCard.selectedCount]]=temp;
					*/

				playedDeck.display(DefaultPlayedDeck_x, DefaultPlayedDeck_y);
				player[turn].rackCard.show();
				player[turn].picked=true;//no cancel
			}
		}
	}
	return listenMenu();	
}

int CControl::doSelectLetter()
{
	int choice;
	showMessage(SELECTLETTER);
	choice=listener.listenIndex();
	if (choice<=7&&choice>=1) 
	{
		if (player[turn].selectLetter(choice-1))
		{
			message.showWord(player[turn].rackLetter.tokens[choice-1].getValue());
		}
	}
	else
	{
		showMessage(INVALIDINDEXENTERED);
		return doSelectLetter();
	}
	/*
	newAction=listenMenu();
	switch(newAction)
	{
	case SELECTLETTER:
		return doSelectLetter();
		break;
	case CANCEL:
		player[turn].updateLetters(true);
		break;
	case DISCARD:
		//add some prompted warning message in future.
		for (i=0; i<player[turn].rackLetter.selectedCount; i++)
		{
			CToken temp;
			player[turn].rackLetter.removeToken(player[turn].rackLetter.selected[i], temp);
			bag.addLetter(temp);
		}
		player[turn].rackLetter.updateRack();//no turning back!!!
		bag.show();
		//maybe deck need refresh for displaying
		return ENDROUND;//this is considered to be end of round 
		break;
	default:
		return newAction;
	}
	*/
	return listenMenu();
}

int CControl::doSelectCard()
{	
	int choice;
	showMessage(SELECTCARD);
	choice=listener.listenIndex();
	if (choice<=7&&choice>=1) 
	{
		player[turn].selectCard(choice-1);
	}
	else
	{
		showMessage(INVALIDINDEXENTERED);//prompt user to re-enter choice
		return doSelectCard();
	}

	return listenMenu();
}

int CControl::doTrade()
{
	//int points;
	showMessage(TRADE);

	if (!player[turn].verifyCards())
	{
		showMessage(INVALIDCARDSTRADING);//warn user to reselect cards
		player[turn].updateCards(true);
		return listenMenu();//rollback
	}
	else
	{
		CToken temp;
		char ch;
		showMessage(INPUTEXCHANGELETTER);
		ch=listenChar();
		ch=toupper(ch);
		if (!isalpha(ch))
		{
			player[turn].updateCards(true);
			return listenMenu();
		}
		//need some verifications here!!!!
		if (bag.removeLetter(ch, temp))
		{
			player[turn].addExchanged(ch);
			message.showWord(ch);
			player[turn].traded=true;
		}
		else
		{
			showMessage(CANNOTEXCHANGELETTER);//warn user to reselect cards
			player[turn].updateCards(true);				
		}			
	}
	return listenMenu();
}


void CControl::showMessage(int msgID)
{
	message.showMessage(msgID);
}

int CControl::listenMenu()
{
	int result=-1;
	while (result<1||result>9)
	{
		showMessage(WAITINGMENUCHOICE);
		result= listener.listenMenu();
	}
	menu.setSelected(result-1);
	return result;
}

void CControl::show()
{
	player[turn].show();
	playingDeck.show();
	playedDeck.show();
	board.show();
	bag.show();
	menu.show();
	message.show();
	scoreBoard.show();
	//scoreboard.show(); //this is for Charles
}

void CControl::fillUpRack()
{
	CToken temp;
	for (int i=0; i<player[turn].rackCard.selectedCount; i++)
	{
		temp=player[turn].rackCard.tokens[player[turn].rackCard.selected[i]];
		if (temp.getValue()!=0)
		{
			playedDeck.addCard(temp);
		}
		if (playingDeck.removeCard(temp))
		{
			player[turn].rackCard.tokens[player[turn].rackCard.selected[i]]=temp;
		}
	}
	//player[turn].rackCard.updateRack();
	player[turn].rackCard.selectedCount=0;
	player[turn].rackCard.show();
	playingDeck.show();
	playedDeck.show();
	for (i=0; i<player[turn].rackLetter.selectedCount; i++)
	{
		temp=player[turn].rackLetter.tokens[player[turn].rackLetter.selected[i]];
		//for the initialization
		if (temp.getValue()!=0)
		{
			bag.addLetter(temp);		
		}
	}
	bag.shuffle();
	for (i=0; i<player[turn].rackLetter.selectedCount; i++)
	{
		if (bag.removeLetter(temp))
		{
			player[turn].rackLetter.tokens[player[turn].rackLetter.selected[i]]=temp;
		}
	}

	//player[turn].rackLetter.updateRack();
	player[turn].rackLetter.selectedCount=0;
	bag.show();
	player[turn].rackLetter.show();
}

void CControl::initPlayGame()
{
	backGround.show();
	if (!dictionary.readFile(fileName))
	{
		showMessage(INVALIDDICTIONARYFILENAME);
		showMessage(TRYUSEDEFAULTFILENAME);
		fileName=defaultInputFileName;
		if (!dictionary.readFile(fileName))
		{
			showMessage(INVALIDDICTIONARYFILENAME);
			exit(INVALIDDICTIONARYFILENAME);
		}
	}
	
	//dictionary.readFile(defaultInputFileName);
	turn=0;
	playingDeck.setFace(true);
	playingDeck.setCoord(DefaultPlayingDeck_x, DefaultPlayingDeck_y);
	playedDeck.setFace(false);
	playedDeck.setCoord(DefaultPlayedDeck_x, DefaultPlayedDeck_y);
	fillUpRack();
	player[turn].traded=false;
	player[turn].formed=false;
	player[turn].picked=false;
	nextTurn();
	fillUpRack();
	player[turn].traded=false;
	player[turn].formed=false;
	player[turn].picked=false;
	nextTurn();
	menu.setSelected(-1);
	show();
}

void CControl::setInputFile(char* inputFileName)
{
	if (inputFileName!=NULL)
	{
		fileName=inputFileName;
	}
}

CControl::CControl()
{
	fileName=defaultInputFileName;
	backGround.show();
}

void CControl::getPlayerName(int index)
{
	char* ptr;
	if (index==0)
	{
		showMessage(INPUTFIRSTPLAYERNAME);
	}
	else
	{
		showMessage(INPUTSECONDPLAYERNAME);
	}
	ptr=listener.listenStr();
	if (strlen(ptr)>DefaultScoreBoardWidth)
	{
		ptr[DefaultScoreBoardWidth]='\0';
	}
	message.showString(ptr);

	strcpy(playerName[index], ptr);
	
}

void CControl::startGame()
{
	backGround.show();
	getPlayerName(0);
	getPlayerName(1);
	scoreBoard.setPlayerName(playerName[0], playerName[1]);
	//debug
	//scoreBoard.show();
	showMessage(PRESSANYKEYTOSTART);
	listener.listenMenu();
}


void CControl::startLogo()
{
	backGround.show();
	logo.show();
	Sleep(3000);
}

int CControl::listenInt()
{
	int result;
	result=listener.listenInt();
	message.showInt(result);
	message.showChar(',');
	return result;
}

char CControl::listenChar()
{
	char ch;
	ch=listener.listenChar();
	message.showChar(ch);
	message.showChar(',');
	return ch;
}



