#include <stdio.h>
#include "dictionary.h"

const int MinimumLengthOfStr=2;

void CDictionary::change2Capital(char* target, char* source)
{
	char* tgt=target, *src=source;
	do
	{
		*tgt=toupper(*src);
		tgt++;
		src++;
	}while (*src!='\0');
	*tgt='\0';
}

bool CDictionary::readFile(const char* fileName)
{
	char buffer[MaxLengthOfWord];
	char*ptr=buffer;
	char ch;
	FILE* stream;
	stream=fopen(fileName, "r");
	if (stream==NULL)
	{
		return false;
	}
	while (!feof(stream))
	{
		ch=fgetc(stream);
		if (isalpha(ch))
		{
			if (!caseSensitive)
			{
				*ptr=toupper(ch);
			}
			else
			{
				*ptr=ch;
			}
			ptr++;
		}
		else
		{
			*ptr='\0';
			if (ptr!=buffer)
			{
				if (strlen(buffer)>=MinimumLengthOfStr)
				{
					addWord(buffer);					
				}
				ptr=buffer;
			
			}
			//else remain the same ptr.
		}
	}
	fclose(stream);
	return true;
}


bool CDictionary::findWord(char* str)
{
	char* ptr=str;
	char buffer[MaxLengthOfWord];
	if (str==NULL)
	{
		return false;
	}

	if (!caseSensitive)
	{		
		ptr=buffer;
		change2Capital(buffer, str);
	}
	Letter* hold;
	int index;
	//not empty string
	if (ptr!=NULL)
	{
		index = indexOf(*ptr);
		if (index==-1)
		{
			return false;
		}
		current=root[index];		
		ptr++;

		if (current->son==NULL)
		{
			//if string is a one-letter string
			if (*ptr=='\0')
			{
				//and there is a one-letter word in dictionary
				return current->end;
			}
			else
			{
				return false;
			}
		}
		hold=current;//
		current=current->son;
		
		while (*ptr!='\0')
		{				
			current=findBrother(current, *ptr);
			if (current==NULL)
			{
				return false;
			}
			if (current->ch==*ptr)
			{
				hold=current;
				current=current->son;
			}
			else
			{
				//not found
				return false;
			}
			ptr++;
		} 
		return hold->end;
	}
	//in my dictionary there is no empty string word
	return false;
}
	

Letter* CDictionary::createLetter(char ch)
{
	Letter* ptr=new Letter;
	ptr->ch=ch;
	ptr->end=false;
	ptr->next=NULL;
	ptr->son=NULL;
	return ptr;
}

//ch is not '\0'
Letter* CDictionary::findBrother(Letter* brother, char ch)
{
	Letter* hold=brother;
	if (brother==NULL)
	{
		return NULL;
	}
	while (hold->next!=NULL)
	{
		if (hold->ch==ch)
		{
			break;
		}
		hold=hold->next;
	}
	return hold;
}

Letter* CDictionary::addSon(char ch)
{
	//the word ends
	if (ch=='\0')
	{
		current->end=true;		
	}
	else
	{
		//need a new son
		if (current->son==NULL)
		{
			current->son=createLetter(ch);
			current=current->son;
		}
		else
		{
			//current->son is not NULL!!!
			current=findBrother(current->son, ch);
			//check if the current is the node
			if (current->ch!=ch)
			{
				current->next=createLetter(ch);	
				current=current->next;//add brother
			}	
			//else return current;!!!
		}
	}
	return current;
}


//add word actually add letter by letter till NULL, nonsense!
bool CDictionary::addWord(char* str)
{
	char* ptr=str;
	int index;
	if (*ptr!='\0')
	{
		index=indexOf(*ptr);
		if (index==-1)
		{
			return false;
		}
		current=root[index];		
		do
		{
			ptr++;
			current=addSon(*ptr);			
		} while (*ptr!='\0');
		return true;
	}
	return false;
}

CDictionary::CDictionary(bool isCaseSensitive)
{
	caseSensitive=isCaseSensitive;

	for (int i=0; i<LetterCount; i++)
	{
		root[i]=new Letter;
		root[i]->ch='A'+i;
		root[i]->next=NULL;
		root[i]->son=NULL;
		root[i]->end=false;
	}
	if (caseSensitive)
	{
		for (i=0; i<LetterCount; i++)
		{
			root[i+LetterCount]=new Letter;
			root[i+LetterCount]->ch='a'+i;
			root[i+LetterCount]->next=NULL;
			root[i+LetterCount]->son=NULL;
		}
	}
}

int CDictionary::indexOf(char ch)
{
	if (ch-'A'>=0&&ch-'Z'<=0)
	{
		return ch-'A';
	}
	if (ch-'a'>=0&&ch-'z'<=0)
	{
		return ch-'a'+LetterCount;
	}
	return -1;
}


